from __future__ import unicode_literals

from daydreamer.tests.views.core import http


class TestCase(http.TestCase):
    """
    Common utilities for testing vary view behaviors.
    
    """
    pass
