from __future__ import unicode_literals

from daydreamer.tests.views.core import behaviors


class TestCase(behaviors.TestCase):
    """
    Common utilities for testing authentication view behaviors.
    
    """
    pass
